@@ -23,6 +23,7 @@
 /* The goal is to make the implementation transparent, so that you
    don't have to know what data types are used, just what functions
    you can call.  I think I have done that. */
+#define READLINE_LIBRARY
 
 #include <stdio.h>
 #include <sys/types.h>
@@ -72,8 +73,8 @@ extern char *strcpy ();
 #define whitespace(c) (((c) == ' ') || ((c) == '\t'))
 #endif
 
-#ifndef digit
-#define digit(c)  ((c) >= '0' && (c) <= '9')
+#ifndef digit_p
+#define digit_p(c)  ((c) >= '0' && (c) <= '9')
 #endif
 
 #ifndef digit_value
@@ -117,7 +118,7 @@ static HIST_ENTRY **the_history = (HIST_ENTRY **)NULL;
 
 /* Non-zero means that we have enforced a limit on the amount of
    history that we save. */
-int history_stifled = 0;
+static int history_stifled = 0;
 
 /* If HISTORY_STIFLED is non-zero, then this is the maximum number of
    entries to remember. */
@@ -168,6 +169,9 @@ history_get_history_state ()
   state->offset = history_offset;
   state->length = history_length;
   state->size = history_size;
+  state->flags = 0;
+  if (history_stifled)
+    state->flags |= HS_STIFLED;
 
   return (state);
 }
@@ -181,6 +185,8 @@ history_set_history_state (state)
   history_offset = state->offset;
   history_length = state->length;
   history_size = state->size;
+  if (state->flags & HS_STIFLED)
+    history_stifled = 1;
 }
 
 /* Begin a session in which the history functions might be used.  This
@@ -490,6 +496,12 @@ unstifle_history ()
   return (result);
 }
 
+int
+history_is_stifled ()
+{
+  return (history_stifled);
+}
+
 /* Return the string that should be used in the place of this
    filename.  This only matters when you don't specify the
    filename to read_history (), or write_history (). */
@@ -664,7 +676,7 @@ history_truncate_file (fname, lines)
 
   /* Write only if there are more lines in the file than we want to
      truncate to. */
-  if (i && ((file = open (filename, O_WRONLY, 0666)) != -1))
+  if (i && ((file = open (filename, O_WRONLY|O_TRUNC, 0666)) != -1))
     {
       write (file, buffer + i, finfo.st_size - i);
       close (file);
@@ -922,10 +934,10 @@ get_history_event (string, caller_index, delimiting_quote)
       i++;
     }
 
-  if (digit (string[i]))
+  if (digit_p (string[i]))
     {
       /* Get the extent of the digits and compute the value. */
-      for (which = 0; digit (string[i]); i++)
+      for (which = 0; digit_p (string[i]); i++)
 	which = (which * 10) + digit_value (string[i]);
 
       *caller_index = i;
@@ -1491,7 +1503,7 @@ history_expand_internal (string, start, end_index_ptr, ret_string, current_line)
 
   n = strlen (temp);
   if (n > result_len)
-    result = xrealloc (result, n + 1);
+    result = xrealloc (result, n + 2);
   strcpy (result, temp);
   free (temp);
 
@@ -1518,8 +1530,12 @@ history_expand_internal (string, start, end_index_ptr, ret_string, current_line)
 	  { \
 	    int sl = strlen (s); \
 	    j += sl; \
-	    while (j >= result_len) \
-	      result = xrealloc (result, result_len += 128); \
+	    if (j >= result_len) \
+	      { \
+	        while (j >= result_len) \
+	          result_len += 128; \
+	        result = xrealloc (result, result_len); \
+	      } \
 	    strcpy (result + j - sl, s); \
 	  } \
 	while (0)
@@ -1527,7 +1543,7 @@ history_expand_internal (string, start, end_index_ptr, ret_string, current_line)
 #define ADD_CHAR(c) \
 	do \
 	  { \
-	    if (j >= result_len) \
+	    if (j >= result_len - 1) \
 	      result = xrealloc (result, result_len += 64); \
 	    result[j++] = c; \
 	    result[j] = '\0'; \
@@ -1550,6 +1566,14 @@ history_expand (hstring, output)
   /* Used when adding the string. */
   char *temp;
 
+  /* Setting the history expansion character to 0 inhibits all
+     history expansion. */
+  if (history_expansion_char == 0)
+    {
+      *output = savestring (hstring);
+      return (0);
+    }
+    
   /* Prepare the buffer for printing error messages. */
   result = xmalloc (result_len = 256);
   result[0] = '\0';
@@ -1795,11 +1819,13 @@ get_history_word_specifier (spec, from, caller_index)
 
   /* Try to get FIRST and LAST figured out. */
 
-  if (spec[i] == '-' || spec[i] == '^')
+  if (spec[i] == '-')
+    first = 0;
+  else if (spec[i] == '^')
     first = 1;
-  else if (digit (spec[i]) && expecting_word_spec)
+  else if (digit_p (spec[i]) && expecting_word_spec)
     {
-      for (first = 0; digit (spec[i]); i++)
+      for (first = 0; digit_p (spec[i]); i++)
 	first = (first * 10) + digit_value (spec[i]);
     }
   else
@@ -1816,9 +1842,9 @@ get_history_word_specifier (spec, from, caller_index)
     {
       i++;
 
-      if (digit (spec[i]))
+      if (digit_p (spec[i]))
 	{
-	  for (last = 0; digit (spec[i]); i++)
+	  for (last = 0; digit_p (spec[i]); i++)
 	    last = (last * 10) + digit_value (spec[i]);
 	}
       else if (spec[i] == '$')
