@@ -53,9 +53,9 @@ extern int errno;
 /* System-specific feature definitions and include files. */
 #include "rldefs.h"
 
-#if defined (WINSIZE_IN_IOCTL_H)
+#if defined (GWINSZ_IN_SYS_IOCTL) || (defined (VSTATUS) && !defined (SunOS4))
 #  include <sys/ioctl.h>
-#endif /* WINSIZE_IN_IOCTL_H */
+#endif /* GWINSZ_IN_SYS_IOCTL || VSTATUS */
 
 /* Some standard library routines. */
 #include "readline.h"
@@ -74,12 +74,14 @@ extern void _rl_bind_if_unbound ();
 
 /* External redisplay functions and variables from display.c */
 extern void _rl_move_vert ();
+extern void _rl_update_final ();
 
 extern void _rl_erase_at_end_of_line ();
 extern void _rl_move_cursor_relative ();
 
 extern int _rl_vis_botlin;
 extern int _rl_last_c_pos;
+extern int _rl_horizontal_scroll_mode;
 extern int rl_display_fixed;
 extern char *rl_display_prompt;
 
@@ -106,6 +108,7 @@ static char *readline_internal ();
 static void readline_initialize_everything ();
 static int init_terminal_io ();
 static void start_using_history ();
+static void bind_arrow_keys ();
 
 #if !defined (__GO32__)
 static void readline_default_bindings ();
@@ -224,6 +227,10 @@ char *rl_line_buffer = (char *)NULL;
 int rl_line_buffer_len = 0;
 #define DEFAULT_BUFFER_SIZE 256
 
+/* Forward declarations used by the display and termcap code. */
+int term_xn;
+int screenwidth, screenheight, screenchars;
+
 
 /* **************************************************************** */
 /*								    */
@@ -914,6 +921,8 @@ _rl_kill_kbd_macro ()
 /* Initliaze readline (and terminal if not already). */
 rl_initialize ()
 {
+  char *t;
+
   /* If we have never been called before, initialize the
      terminal and data structures. */
   if (!rl_initialized)
@@ -930,6 +939,16 @@ rl_initialize ()
   /* We aren't done yet.  We haven't even gotten started yet! */
   rl_done = 0;
 
+  /* Check for LC_CTYPE and use its value to decide the defaults for
+     8-bit character input and output. */
+  t = getenv ("LC_CTYPE");
+  if (t && (strcmp (t, "iso-8859-1") == 0 || strcmp (t, "iso_8859_1") == 0))
+    {
+      _rl_meta_flag = 1;
+      _rl_convert_meta_chars_to_ascii = 0;
+      _rl_output_meta_chars = 1;
+    }
+      
   /* Tell the history routines what is going on. */
   start_using_history ();
 
@@ -982,10 +1001,20 @@ readline_initialize_everything ()
   /* Read in the init file. */
   rl_read_init_file ((char *)NULL);
 
+  /* XXX */
+  if (_rl_horizontal_scroll_mode && term_xn)
+    {
+      screenwidth--;
+      screenchars -= screenheight;
+    }
+
   /* Override the effect of any `set keymap' assignments in the
      inputrc file. */
   rl_set_keymap_from_edit_mode ();
 
+  /* Try to bind a common arrow key prefix, if not already bound. */
+  bind_arrow_keys ();
+
   /* If the completion parser's default word break characters haven't
      been set yet, then do so now. */
   if (rl_completer_word_break_characters == (char *)NULL)
@@ -1001,6 +1030,51 @@ readline_default_bindings ()
   rltty_set_default_bindings (_rl_keymap);
 }
 
+static void
+bind_arrow_keys_internal ()
+{
+  Function *f;
+
+  f = rl_function_of_keyseq ("\033[A", _rl_keymap, (int *)NULL);
+  if (!f || f == rl_do_lowercase_version)
+    {
+      _rl_bind_if_unbound ("\033[A", rl_get_previous_history);
+      _rl_bind_if_unbound ("\033[B", rl_get_next_history);
+      _rl_bind_if_unbound ("\033[C", rl_forward);
+      _rl_bind_if_unbound ("\033[D", rl_backward);
+    }
+
+  f = rl_function_of_keyseq ("\033OA", _rl_keymap, (int *)NULL);
+  if (!f || f == rl_do_lowercase_version)
+    {
+      _rl_bind_if_unbound ("\033OA", rl_get_previous_history);
+      _rl_bind_if_unbound ("\033OB", rl_get_next_history);
+      _rl_bind_if_unbound ("\033OC", rl_forward);
+      _rl_bind_if_unbound ("\033OD", rl_backward);
+    }
+}
+
+/* Try and bind the common arrow key prefix after giving termcap and
+   the inputrc file a chance to bind them and create `real' keymaps
+   for the arrow key prefix. */
+static void
+bind_arrow_keys ()
+{
+  Keymap xkeymap;
+
+  xkeymap = _rl_keymap;
+
+  _rl_keymap = emacs_standard_keymap;
+  bind_arrow_keys_internal ();
+
+#if defined (VI_MODE)
+  _rl_keymap = vi_movement_keymap;
+  bind_arrow_keys_internal ();
+#endif
+
+  _rl_keymap = xkeymap;
+}
+
 
 /* **************************************************************** */
 /*								    */
@@ -1026,7 +1100,7 @@ rl_digit_loop ()
 	  continue;
 	}
       c = UNMETA (c);
-      if (numeric (c))
+      if (digit_p (c))
 	{
 	  if (rl_explicit_arg)
 	    rl_numeric_arg = (rl_numeric_arg * 10) + (c - '0');
@@ -1103,16 +1177,14 @@ int dumb_term = 0;
 #undef PC
 
 #if !defined (__linux__)
-char PC;
-char *BC, *UP;
+/* If this causes problems, add back the `extern'. */
+/*extern*/ char PC, *BC, *UP;
 #endif /* __linux__ */
 
 /* Some strings to control terminal actions.  These are output by tputs (). */
 char *term_goto, *term_clreol, *term_cr, *term_clrpag, *term_backspace;
 char *term_pc;
 
-int screenwidth, screenheight, screenchars;
-
 /* Non-zero if we determine that the terminal can do character insertion. */
 int terminal_can_insert = 0;
 
@@ -1215,7 +1287,8 @@ _rl_set_screen_size (tty, ignore_env)
   set_lines_and_columns (screenheight, screenwidth);
 #endif
 
-  screenwidth--;
+  if (!term_xn)
+    screenwidth--;
 
   screenchars = screenwidth * screenheight;
 }
@@ -1292,7 +1365,7 @@ init_terminal_io (terminal_name)
 #if defined (HACK_TERMCAP_MOTION)
   term_forward_char = (char *)NULL;
 #endif /* HACK_TERMCAP_MOTION */
-  terminal_can_insert = 0;
+  terminal_can_insert = term_xn = 0;
   return;
 #else /* !__GO32__ */
 
@@ -1350,6 +1423,8 @@ init_terminal_io (terminal_name)
 
   screenwidth = screenheight = 0;
 
+  term_xn = tgetflag ("am") && tgetflag ("xn");
+
   _rl_set_screen_size (tty, 0);
 
   /* "An application program can assume that the terminal can do
@@ -1451,6 +1526,18 @@ crlf ()
   return 0;
 }
 
+rl_tty_status (count, key)
+     int count, key;
+{
+#if defined (TIOCSTAT)
+  ioctl (1, TIOCSTAT, (char *)0);
+  rl_refresh_line ();
+#else
+  ding ();
+#endif
+  return 0;
+}
+
 
 /* **************************************************************** */
 /*								    */
@@ -1468,7 +1555,7 @@ int
 alphabetic (c)
      int c;
 {
-  if (pure_alphabetic (c) || (numeric (c)))
+  if (pure_alphabetic (c) || (digit_p (c)))
     return (1);
 
   if (allow_pathname_alphabetic_chars)
@@ -1477,14 +1564,6 @@ alphabetic (c)
     return (0);
 }
 
-/* Return non-zero if C is a numeric character. */
-int
-numeric (c)
-     int c;
-{
-  return (c >= '0' && c <= '9');
-}
-
 /* Ring the terminal bell. */
 int
 ding ()
@@ -1519,7 +1598,8 @@ ding ()
 }
 
 /* How to abort things. */
-rl_abort ()
+rl_abort (count, key)
+     int count, key;
 {
   ding ();
   rl_clear_message ();
@@ -1683,8 +1763,8 @@ rl_delete_text (from, to)
    might as well let rl_redisplay do that job. */
 
 /* Move forward COUNT characters. */
-rl_forward (count)
-     int count;
+rl_forward (count, key)
+     int count, key;
 {
   if (count < 0)
     rl_backward (-count);
@@ -1709,8 +1789,8 @@ rl_forward (count)
 }
 
 /* Move backward COUNT characters. */
-rl_backward (count)
-     int count;
+rl_backward (count, key)
+     int count, key;
 {
   if (count < 0)
     rl_forward (-count);
@@ -1728,22 +1808,24 @@ rl_backward (count)
 }
 
 /* Move to the beginning of the line. */
-rl_beg_of_line ()
+rl_beg_of_line (count, key)
+     int count, key;
 {
   rl_point = 0;
   return 0;
 }
 
 /* Move to the end of the line. */
-rl_end_of_line ()
+rl_end_of_line (count, key)
+     int count, key;
 {
   rl_point = rl_end;
   return 0;
 }
 
 /* Move forward a word.  We do what Emacs does. */
-rl_forward_word (count)
-     int count;
+rl_forward_word (count, key)
+     int count, key;
 {
   int c;
 
@@ -1784,8 +1866,8 @@ rl_forward_word (count)
 }
 
 /* Move backward a word.  We do what Emacs does. */
-rl_backward_word (count)
-     int count;
+rl_backward_word (count, key)
+     int count, key;
 {
   int c;
 
@@ -1870,7 +1952,8 @@ rl_refresh_line ()
 /* C-l typed to a line without quoting clears the screen, and then reprints
    the prompt and the current input line.  Given a numeric arg, redraw only
    the current line. */
-rl_clear_screen ()
+rl_clear_screen (count, key)
+     int count, key;
 {
   if (rl_explicit_arg)
     {
@@ -2011,19 +2094,18 @@ rl_insert (count, c)
 }
 
 /* Insert the next typed character verbatim. */
-rl_quoted_insert (count)
-     int count;
+rl_quoted_insert (count, key)
+     int count, key;
 {
   int c;
 
   c = rl_read_key ();
-  return (rl_insert (count, c));
-  
+  return (rl_insert (count, c));  
 }
 
 /* Insert a tab character. */
-rl_tab_insert (count)
-     int count;
+rl_tab_insert (count, key)
+     int count, key;
 {
   return (rl_insert (count, '\t'));
 }
@@ -2043,13 +2125,7 @@ rl_newline (count, key)
 #endif /* VI_MODE */
 
   if (readline_echoing_p)
-    {
-      _rl_move_vert (_rl_vis_botlin);
-      _rl_vis_botlin = 0;
-      crlf ();
-      fflush (out_stream);
-      rl_display_fixed++;
-    }
+    _rl_update_final ();
   return 0;
 }
 
@@ -2076,8 +2152,8 @@ rl_do_lowercase_version (ignore1, ignore2)
 }
 
 /* Rubout the character behind point. */
-rl_rubout (count)
-     int count;
+rl_rubout (count, key)
+     int count, key;
 {
   if (count < 0)
     {
@@ -2176,7 +2252,8 @@ rl_delete_horizontal_space (count, ignore)
 
 /* This does what C-w does in Unix.  We can't prevent people from
    using behaviour that they expect. */
-rl_unix_word_rubout ()
+rl_unix_word_rubout (count, key)
+     int count, key;
 {
   if (!rl_point)
     ding ();
@@ -2201,7 +2278,8 @@ rl_unix_word_rubout ()
    latter, because if you are a Unix weenie, then you haven't backspaced
    into the line at all, and if you aren't, then you know what you are
    doing. */
-rl_unix_line_discard ()
+rl_unix_line_discard (count, key)
+     int count, key;
 {
   if (!rl_point)
     ding ();
@@ -2236,22 +2314,22 @@ rl_unix_line_discard ()
 static int rl_change_case ();
 
 /* Uppercase the word at point. */
-rl_upcase_word (count)
-     int count;
+rl_upcase_word (count, key)
+     int count, key;
 {
   return (rl_change_case (count, UpCase));
 }
 
 /* Lowercase the word at point. */
-rl_downcase_word (count)
-     int count;
+rl_downcase_word (count, key)
+     int count, key;
 {
   return (rl_change_case (count, DownCase));
 }
 
 /* Upcase the first letter, downcase the rest. */
-rl_capitalize_word (count)
-     int count;
+rl_capitalize_word (count, key)
+     int count, key;
 {
  return (rl_change_case (count, CapCase));
 }
@@ -2323,8 +2401,8 @@ rl_change_case (count, op)
 /* **************************************************************** */
 
 /* Transpose the words at point. */
-rl_transpose_words (count)
-     int count;
+rl_transpose_words (count, key)
+     int count, key;
 {
   char *word1, *word2;
   int w1_beg, w1_end, w2_beg, w2_end;
@@ -2383,8 +2461,8 @@ rl_transpose_words (count)
 
 /* Transpose the characters at point.  If point is at the end of the line,
    then transpose the characters before point. */
-rl_transpose_chars (count)
-     int count;
+rl_transpose_chars (count, key)
+     int count, key;
 {
   char dummy[2];
 
@@ -2522,6 +2600,7 @@ undo_thing:
 }
 
 /* Begin a group.  Subsequent undos are undone as an atomic operation. */
+int
 rl_begin_undo_group ()
 {
   rl_add_undo (UNDO_BEGIN, 0, 0, 0);
@@ -2529,6 +2608,7 @@ rl_begin_undo_group ()
 }
 
 /* End an undo group started with rl_begin_undo_group (). */
+int
 rl_end_undo_group ()
 {
   rl_add_undo (UNDO_END, 0, 0, 0);
@@ -2558,7 +2638,9 @@ rl_modifying (start, end)
 }
 
 /* Revert the current line to its previous state. */
-rl_revert_line ()
+int
+rl_revert_line (count, key)
+     int count, key;
 {
   if (!rl_undo_list)
     ding ();
@@ -2571,8 +2653,9 @@ rl_revert_line ()
 }
 
 /* Do some undoing of things that were done. */
-rl_undo_command (count)
-     int count;
+int
+rl_undo_command (count, key)
+     int count, key;
 {
   if (count < 0)
     return 0;	/* Nothing to do. */
@@ -2684,13 +2767,15 @@ maybe_save_line ()
 /* **************************************************************** */
 
 /* Meta-< goes to the start of the history. */
-rl_beginning_of_history ()
+rl_beginning_of_history (count, key)
+     int count, key;
 {
   return (rl_get_previous_history (1 + where_history ()));
 }
 
 /* Meta-> goes to the end of the history.  (The current line). */
-rl_end_of_history ()
+rl_end_of_history (count, key)
+     int count, key;
 {
   maybe_replace_line ();
   using_history ();
@@ -2699,8 +2784,8 @@ rl_end_of_history ()
 }
 
 /* Move down to the next history line. */
-rl_get_next_history (count)
-     int count;
+rl_get_next_history (count, key)
+     int count, key;
 {
   HIST_ENTRY *temp = (HIST_ENTRY *)NULL;
 
@@ -2744,8 +2829,8 @@ rl_get_next_history (count)
 
 /* Get the previous item out of our interactive history, making it the current
    line.  If there is no previous history, just ding. */
-rl_get_previous_history (count)
-     int count;
+rl_get_previous_history (count, key)
+     int count, key;
 {
   HIST_ENTRY *old_temp = (HIST_ENTRY *)NULL;
   HIST_ENTRY *temp = (HIST_ENTRY *)NULL;
@@ -2826,7 +2911,8 @@ rl_set_mark (position)
 }
 
 /* Exchange the position of mark and point. */
-rl_exchange_mark_and_point ()
+rl_exchange_mark_and_point (count, key)
+     int count, key;
 {
   if (rl_mark > rl_end)
     rl_mark = -1;
@@ -2973,8 +3059,8 @@ rl_kill_text (from, to)
 /* **************************************************************** */
 
 /* Delete the word at point, saving the text in the kill ring. */
-rl_kill_word (count)
-     int count;
+rl_kill_word (count, key)
+     int count, key;
 {
   int orig_point = rl_point;
 
@@ -3012,8 +3098,8 @@ rl_backward_kill_word (count, ignore)
 
 /* Kill from here to the end of the line.  If DIRECTION is negative, kill
    back to the line start instead. */
-rl_kill_line (direction)
-     int direction;
+rl_kill_line (direction, ignore)
+     int direction, ignore;
 {
   int orig_point = rl_point;
 
@@ -3031,8 +3117,8 @@ rl_kill_line (direction)
 
 /* Kill backwards to the start of the line.  If DIRECTION is negative, kill
    forwards to the line end instead. */
-rl_backward_kill_line (direction)
-     int direction;
+rl_backward_kill_line (direction, ignore)
+     int direction, ignore;
 {
   int orig_point = rl_point;
 
@@ -3154,6 +3240,19 @@ rl_yank_nth_arg (count, ignore)
   return 0;
 }
 
+/* Yank the last argument from the previous history line.  This `knows'
+   how rl_yank_nth_arg treats a count of `$'.  With an argument, this
+   behaves the same as rl_yank_nth_arg. */
+int
+rl_yank_last_arg (count, key)
+     int count, key;
+{
+  if (rl_explicit_arg)
+    return (rl_yank_nth_arg (count, key));
+  else
+    return (rl_yank_nth_arg ('$', key));
+}
+
 /* How to toggle back and forth between editing modes. */
 rl_vi_editing_mode (count, key)
      int count, key;
@@ -3253,13 +3352,71 @@ rl_getc (stream)
 #endif
 /* Backwards compatibilty, now that savestring has been removed from
    all `public' readline header files. */
+char *
 savestring (s)
      char *s;
 {
-  return ((char *)strcpy (xmalloc (1 + (int)strlen (x)), (x)));
+  return ((char *)strcpy (xmalloc (1 + (int)strlen (s)), (s)));
 }
 #endif
 
+/* Function equivalents for the macros defined in chartypes.h. */
+#undef uppercase_p
+int
+uppercase_p (c)
+     int c;
+{
+  return (isupper (c));
+}
+
+#undef lowercase_p
+int
+lowercase_p (c)
+     int c;
+{
+  return (islower (c));
+}
+
+#undef pure_alphabetic
+int
+pure_alphabetic (c)
+     int c;
+{
+  return (isupper (c) || islower (c));
+}
+
+#undef digit_p
+int
+digit_p (c)
+     int c;
+{
+  return (isdigit (c));
+}
+
+#undef to_lower
+int
+to_lower (c)
+     int c;
+{
+  return (isupper (c) ? tolower (c) : c);
+}
+
+#undef to_upper
+int
+to_upper (c)
+     int c;
+{
+  return (islower (c) ? toupper (c) : c);
+}
+
+#undef digit_value
+int
+digit_value (c)
+     int c;
+{
+  return (isdigit (c) ? c - '0' : c);
+}
+
 #if defined (STATIC_MALLOC)
 
 /* **************************************************************** */
